@@ -10,6 +10,7 @@ export SSO_EXTENSIONS_DIR="/tmp" # this disables all extensions
 
 #Environment variables for the test application
 export G_MESSAGES_DEBUG=all
+export G_SLICE=debug-blocks
 
 TEST_APP=./signon-glib-testsuite
 
@@ -17,7 +18,7 @@ TEST_APP=./signon-glib-testsuite
 # session
 if command -v dbus-test-runner > /dev/null ; then
     echo "Using dbus-test-runner"
-    dbus-test-runner -m 180 -t signond \
+    dbus-test-runner -m 180 -t signond -r \
         -t "$TEST_APP" -f com.google.code.AccountsSSO.SingleSignOn
 else
     echo "Using existing D-Bus session"
